@@ -7,11 +7,11 @@ class NotFound(Exception):
 
 class Redirect(Exception):
     def __init__(self, url, **kwargs):
-        self.res = wsgi.Response('', code=302, headers={'Location': url}, **kwargs)
+        self.res = wsgi.redirect(url, **kwargs)
 
 class PermanentRedirect(Exception):
     def __init__(self, url, **kwargs):
-        self.res = wsgi.Response('', code=301, headers={'Location': url}, **kwargs)
+        self.res = wsgi.redirect(url, permanent=True, **kwargs)
 
 class BasicAuth(Exception):
     def __init__(self):
