@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 import cgi
 import logging
+import marshal
+import base64
 from hashlib import sha1
 from StringIO import StringIO
 from tempfile import TemporaryFile
@@ -9,7 +11,7 @@ from Cookie import SimpleCookie
 from util import cache_property, make_traceback, obj_from_str, list_obj_from_str
 from config import cfg
 from exceptions import NotFound, Redirect, PermanentRedirect, BasicAuth
-from routing import auto_routing, add_apps
+from routing import auto_routing, add_apps, url4
 from multidict import MultiDict, HeaderDict
 
 
@@ -17,6 +19,8 @@ class Request(object):
     charset = 'utf-8'
     encoding_errors = 'ignore'
     max_post_size = None  # максимальный размер post-данных
+    _new_flashes = None
+    _clear_flashes = False
     
     def __init__(self, environ):
         self.environ = environ
@@ -24,7 +28,7 @@ class Request(object):
     
     @cache_property
     def context(self):
-        res = {}
+        res = {'rq': self, 'url4': url4, 'cfg': cfg}
         for func in cfg.CONTEXT_PROCESSORS:
             res.update(func(self))
         return res
@@ -141,13 +145,27 @@ class Request(object):
         if cfg.BASIC_AUTH_DB.get(user) != sha1(passwd).hexdigest():
             raise BasicAuth
         return user
+        
+    @cache_property
+    def flashes(self):
+        ret = []
+        if cfg.FLASH_COOKIE_NAME in self.COOKIES:
+            ret = self.COOKIES[cfg.FLASH_COOKIE_NAME].value
+            ret = marshal.loads(base64.urlsafe_b64decode(ret))
+            self._clear_flashes = True
+        return ret
 
+    def flash(self, msg, level='info'):
+        if not isinstance(msg, unicode):
+            msg = msg.decode(self.charset)
+        self._new_flashes = self._new_flashes or []
+        self._new_flashes.append((msg, level))
 
 
 class Response(object):
     code = 200  # дефолтный код ответа
     charset = 'utf-8'  # дефолтная кодировка
-    encodeg_errors = 'ignore'
+    encoding_errors = 'ignore'
     content_type = 'text/html'  # дефолтный Content-Type
     
     def __init__(self, body='', headers=None, cookies=None, **kwargs):
@@ -168,18 +186,24 @@ class Response(object):
             for cookie in cookies:
                 self.set_cookie(**cookie)
 
-    def set_cookie(self, key, val, **kwargs):
+    def set_cookie(self, key, val, path='/', **kwargs):
         """
         Устанавливаем куку:
             *key        - (unicode) ключ
             *val        - (unicode) значение
-            **expires   - (int) время жизни куки в секундах, дефолт - до закрытия браузера
             **path      - (unicode) uri, '/' - для действия на весь домен
+            **expires   - (int) время жизни куки в секундах, дефолт - до закрытия браузера
             **domain    - (unicode) дефолт - текущий поддомен, '.site.name' - для всех поддоменов
         """
-        self.COOKIES[key.encode(self.charset, self.encodeg_errors)] = val.encode(self.charset, self.encodeg_errors)
+        key = key.encode(self.charset, self.encoding_errors)
+        self.COOKIES[key] = val.encode(self.charset, self.encoding_errors)
+        self.COOKIES[key]['path'] = path
         for k, v in kwargs.iteritems():
             self.COOKIES[key][k] = v
+        return self
+        
+    def delete_cookie(self, key):
+        return self.set_cookie(key, '', expires=-1)
 
     def wsgi(self):
         '''
@@ -187,7 +211,7 @@ class Response(object):
         '''
         status = '%i %s' % (self.code, HTTP_CODES.get(self.code, 'Unknown'))
         if isinstance(self.body, unicode):
-            self.body = self.body.encode(self.charset, self.encodeg_errors)
+            self.body = self.body.encode(self.charset, self.encoding_errors)
         else:
             self.body = str(self.body)
         # добавляем куки в заголовки
@@ -257,6 +281,17 @@ class App(object):
         return body
 
 
+def redirect(url, permanent=False, **kwargs):
+    '''
+    Редирект
+        *url        - url or urlname
+    '''
+    code = 301 if permanent else 302
+    if not url.startswith(('/', 'http://', 'https://')):
+        url = url4(url) or url
+    return Response('', code=code, headers={'Location': url}, **kwargs)
+
+
 HTTP_CODES = {
     100:    'Continue',
     101:    'Switching Protocols',
